// GBA.H - Gameboy Advance General C Header
// ----------------------------------------

// Helpful Type Definitions
// -------------------------------------------

typedef     unsigned char           u8;
typedef     unsigned short int      u16;
typedef     unsigned int            u32;
typedef     unsigned long long int  u64;

typedef     signed char             s8;
typedef     signed short int        s16;
typedef     signed int              s32;
typedef     signed long long int    s64;

// Intel stylie!!
typedef		unsigned int			DWORD;
typedef		int						BOOL;
typedef		unsigned char			BYTE;
typedef		unsigned short int		WORD;

// OAM Object Structure
// -------------------------------------------

struct oamobject
{
	u16 y;
	u16 x;
	u16 tile;
	u16 fx;
};

typedef struct oamobject OAMOBJECT;


// First WORD
#define OBJ_Y_MASK			0xFF		// Y Position Bit mask
#define OBJ_SPECIAL			0x100		// Sprite uses rot/scaling
#define OBJ_SDOUBLE			0x200		// Double Size Flag ( Doubles bounding box for rot scale sprites)
#define OBJ_BLEND			0x400		// Alpha blend
#define OBJ_WINDOW			0x800		// ???
#define OBJ_MOSAIC			0x1000		// Mosaic Mode
#define OBJ_COLOUR_256x1	0x2000		// Object has 256 colours from 1 palette
#define OBJ_COLOUR_16x16	0			// Object has 16 colours from 1 of 16 palettes
#define OBJ_SQUARE			14			// Object is square (1x1 aspect)
#define OBJ_WIDE			14			// Object is wider than tall. (2x1 aspect)
#define OBJ_TALL			14			// Object is taller than wide (1x2 aspect)
#define OBJ_SHAPE(n)		(n << 14)

// Second WORD
#define OBJ_X_MASK			0x1FF		// X Position bit mask
#define OBJ_HFLIP			0x1000
#define OBJ_VFLIP			0x2000
#define OBJ_SIZE_2X			1			// Object is double size
#define OBJ_SIZE_4X			2			// ...
#define OBJ_SIZE_8X			3			// ...
#define OBJ_SIZE(n)			(n << 14)

// Video Controller Registers
// -------------------------------------------

#define VIDC_BASE		0x4000000		// Video Controller Register Base Address
#define VRAM_BASE		0x6000000		// Vram Base Address
#define VRAM_SIZE		0x00010000
#define VRAM_OBJ_BASE	0x06100000
#define VRAM_OBJ_SIZE	0x00007FFF

#define BG_PAL		0x5000000		// Background Palette 256 * 16bits
#define OBJ_PAL		0x5000200		// Object Palette (ditto)
#define INPUT_BASE	0x4000130		// Control pad base register


#define DISPCNT		*(u32*)(VIDC_BASE)			// Display Control
#define DISPSTAT	*(u16*)(VIDC_BASE + 0x4)	// Dislpay Status
#define VCOUNT		*(u16*)(VIDC_BASE + 0x6)	// Vertical Row Count

#define BG0CNT		*(u16*)(VIDC_BASE + 0x8)	// Background Controls
#define BG1CNT     	*(u16*)(VIDC_BASE + 0xA)
#define BG2CNT     	*(u16*)(VIDC_BASE + 0xC)
#define BG3CNT     	*(u16*)(VIDC_BASE + 0xE)

#define BG0X		*(u16*)(VIDC_BASE + 0x10)	// Background Offsets
#define BG0Y		*(u16*)(VIDC_BASE + 0x12)
#define BG1X		*(u16*)(VIDC_BASE + 0x14)	
#define BG1Y		*(u16*)(VIDC_BASE + 0x16)
#define BG2X		*(u16*)(VIDC_BASE + 0x18)
#define BG2Y		*(u16*)(VIDC_BASE + 0x1A)
#define BG3X		*(u16*)(VIDC_BASE + 0x1C)
#define BG3Y		*(u16*)(VIDC_BASE + 0x1E)

#define MOSAIC		*(u16*)(VIDC_BASE + 0x4C)		// Mosaic Control
#define BLDMOD		*(u16*)(VIDC_BASE + 0x50)		// Blending ControlRegister
#define COLEV		*(u16*)(VIDC_BASE + 0x52)
#define COLY	    *(u16*)(VIDC_BASE + 0x54)

#define BLEND_ALPHA		(1 << 6)					// alpha blend between source and dest
#define BLEND_FADEIN	(2 << 6)					// Fade out
#define BLEND_FADEOUT	(3 << 6)					// Fade in

#define BLEND_SRC(n)	(n & 0x3F)					// Blend source BOPPPP
#define BLEND_DEST(n)	( (n & 0x3F) << 8)			// Blend target

// DMA Registers
// --------------------------

#define DMAC_BASE	0x4000000

#define DMA3SAD		*(u32*)(DMAC_BASE + 0xD4)		// DMA3 Source Address
#define DMA3DAD		*(u32*)(DMAC_BASE + 0xD8)		// DMA3 Destination Address
#define DMA3CNT		*(u32*)(DMAC_BASE + 0xDC)		// DMA3 Length Counter

// Register Bits
// -------------------------------------------

#define DISP_MODE(n)	(n & 7)						// Display Mode Number
#define DISP_FBSEL		0x10						// ????
#define DISP_HBLPROC	0x20						// ????
#define DISP_OAMSEQ		0x40						// Oam Sequencial Flag
#define DISP_DISABLE	0x80						// Display Disable
#define DISP_BG0		0x100						// BG 0 Enable
#define DISP_BG1		0x200						// BG 0 Enable
#define DISP_BG2		0x400						// BG 0 Enable
#define DISP_BG3		0x800						// BG 0 Enable
#define DISP_OBJ		0x1000						// Obj Enable
#define DISP_WINDOW(n)	( ( n & 3 ) << 13 )			// Window Select/Enable
#define DISP_OWNABLE	0x8000						// ????

#define STAT_VCOUNT_MASK	(0x000000FF)
#define STAT_VBL_MASK		(1 << 0)
#define STAT_HBL_MASK		(1 << 1)
#define STAT_VC_MASK		(0xFF << 8)
#define STAT_VC(n)			((n & 0x000000FF) << 8)
#define STAT_VBL_IRQ		(1 << 3)
#define STAT_HBL_IRQ		(1 << 4)
#define STAT_IRQ			(1 << 5)

#define BG_PRIORITY(n)	( n & 3 )					// Background Priority
#define BG_CHARBASE(n)	( ( n & 0xC000 ) >> 12 )		// Character Base 0x6000000 + n

#define BG_MOSAIC		0x40						// Mosaic Mode
#define BG_COLOUR_256x1	0x80						// 256 Colours/Tile, 1 Palette
#define BG_COLOUR_16x16	0							// 16 Colours/Tile, 8 Palettes

#define BG_MAPBASE(n)	( ( n & 0xF800) >> 3 )		// Map Base Address 0x6000000 + n

#define BG_WRAP			0x2000						// Wrap Background at edges
#define BG_NOWRAP		0							// Background is transparent at edges

#define BG_T256x256		0							// Map Sizes......
#define BG_T512x256		0x4000
#define BG_T256x512		0x8000
#define BG_T512x512		0xC000
#define BG_M128x128		0x10000
#define BG_M256x256		0x14000
#define BG_M512x512		0x18000
#define BG_M1024x1024	0x1C000

// Control Pad Defines
// -------------------

#define PADREG		*(u16*)(INPUT_BASE)

#define PAD_BUTTONA	1
#define PAD_BUTTONB	2
#define PAD_SELECT	4
#define PAD_START	8
#define PAD_RIGHT	16
#define PAD_LEFT	32
#define PAD_UP		64
#define PAD_DOWN	128
#define PAD_BUTTONL	256
#define PAD_BUTTONR	512

// Helper Macros
// -------------------------------------------
#define RGB(r,g,b) ((((b)>>3)<<10)+(((g)>>3)<<5)+((r)>>3))
